import { WebGPUProgram } from './webgpu_program';
export declare const RELU = "return max(a, 0.0);";
export declare const RELU6 = "return clamp(a, 0.0, 6.0);";
export declare const LINEAR = "return a;";
export declare const ELU = "return (a >= 0.0) ? a : (exp(a) - 1.0);";
export declare const PRELU = "return (a < 0.) ? b * a : a;";
export declare const ELU_VEC4 = "\n  vec4 result;\n\n  result.r = (a.r >= 0.0) ? a.r : (exp(a.r) - 1.0);\n  result.g = (a.g >= 0.0) ? a.g : (exp(a.g) - 1.0);\n  result.b = (a.b >= 0.0) ? a.b : (exp(a.b) - 1.0);\n  result.a = (a.a >= 0.0) ? a.a : (exp(a.a) - 1.0);\n\n  return result;\n";
export declare const RELU_VEC4 = "\n  vec4 result = a * vec4(greaterThanEqual(a, vec4(0.0)));\n  bvec4 isNaN = isnan(a);\n\n  result.r = isNaN.r ? a.r : result.r;\n  result.g = isNaN.g ? a.g : result.g;\n  result.b = isNaN.b ? a.b : result.b;\n  result.a = isNaN.a ? a.a : result.a;\n\n  return result;\n";
export declare const SIGMOID = "return 1.0 / (1.0 + exp(-1.0 * a));";
export declare const ABS = "return abs(a);";
export declare const SQUARE = "return a * a;";
export declare const NEG = "return -a;";
export declare const TANH = "\n  float e2x = exp(-2.0 * abs(a));\n  return sign(a) * (1.0 - e2x) / (1.0 + e2x);\n";
export declare const EXP = "return exp(a);";
export declare const LOG = "if (a < 0.0) return 1.0/0.0;\n  return log(a);";
export declare const TO_INT = "return float(int(a));";
export declare class UnaryOpProgram implements WebGPUProgram {
    outputShape: number[];
    shaderKey: string;
    dispatchLayout: {
        x: number[];
    };
    dispatch: [number, number, number];
    variableNames: string[];
    workPerThread: number;
    workGroupSize: [number, number, number];
    op: string;
    constructor(outputShape: number[], op: string);
    getUserCode(): string;
}
